@@ -31,11 +31,12 @@
 #include <c-ctype.h>
 #include "gssapi.h"
 #include "auth/common.h"
+#include "auth-unix.h"
 #include <gssapi/gssapi.h>
 #include <gssapi/gssapi_ext.h>
 #include <gssapi/gssapi_krb5.h>
 #include <gl/base64.h>
-#include "cfg.h"
+#include "common-config.h"
 
 static gss_cred_id_t glob_creds;
 static gss_OID_set glob_oids;
@@ -263,8 +264,9 @@ static int gssapi_auth_init(void **ctx, void *pool, const char *spnego, const ch
 
 static int gssapi_auth_group(void *ctx, const char *suggested, char *groupname, int groupname_size)
 {
-	groupname[0] = 0;
-	return 0;
+	struct gssapi_ctx_st *pctx = ctx;
+
+	return get_user_auth_group(pctx->username, suggested, groupname, groupname_size);
 }
 
 static int gssapi_auth_user(void *ctx, char *username, int username_size)
@@ -348,6 +350,17 @@ static void gssapi_auth_deinit(void *ctx)
 	talloc_free(ctx);
 }
 
+static void gssapi_group_list(void *pool, void *_additional, char ***groupname, unsigned *groupname_size)
+{
+	gssapi_cfg_st *config = _additional;
+	gid_t min = 0;
+
+	if (config)
+		min = config->gid_min;
+
+	unix_group_list(pool, min, groupname, groupname_size);
+}
+
 const struct auth_mod_st gssapi_auth_funcs = {
 	.type = AUTH_TYPE_GSSAPI,
 	.auth_init = gssapi_auth_init,
@@ -358,6 +371,7 @@ const struct auth_mod_st gssapi_auth_funcs = {
 	.auth_group = gssapi_auth_group,
 	.global_init = gssapi_global_init,
 	.global_deinit = gssapi_global_deinit,
+	.group_list = gssapi_group_list
 };
 
 #endif
