@@ -24,7 +24,8 @@
 #include <syslog.h>
 #include <vpn.h>
 #include "pam.h"
-#include "cfg.h"
+#include "common-config.h"
+#include "auth-unix.h"
 #include <sec-mod-auth.h>
 #include <ccan/hash/hash.h>
 
@@ -45,6 +46,7 @@
 #include <pwd.h>
 #include <grp.h>
 #include "auth/pam.h"
+#include "auth-unix.h"
 
 #define PAM_STACK_SIZE (96*1024)
 
@@ -273,50 +275,9 @@ struct pam_ctx_st * pctx = ctx;
  */
 static int pam_auth_group(void* ctx, const char *suggested, char *groupname, int groupname_size)
 {
-struct passwd * pwd;
-struct pam_ctx_st * pctx = ctx;
-struct group *grp;
-int ret;
-unsigned found;
+	struct pam_ctx_st * pctx = ctx;
 
-	groupname[0] = 0;
-
-	pwd = getpwnam(pctx->username);
-	if (pwd != NULL) {
-		if (suggested != NULL) {
-			gid_t groups[MAX_GROUPS];
-			int ngroups = sizeof(groups)/sizeof(groups[0]);
-			unsigned i;
-
-			ret = getgrouplist(pctx->username, pwd->pw_gid, groups, &ngroups);
-			if (ret <= 0) {
-				return 0;
-			}
-
-			found = 0;
-			for (i=0;i<ngroups;i++) {
-				grp = getgrgid(groups[i]);
-				if (grp != NULL && strcmp(suggested, grp->gr_name) == 0) {
-					strlcpy(groupname, grp->gr_name, groupname_size);
-					found = 1;
-					break;
-				}
-			}
-
-			if (found == 0) {
-				syslog(LOG_AUTH,
-				       "user '%s' requested group '%s' but is not a member",
-				       pctx->username, suggested);
-				return -1;
-			}
-		} else {
-			struct group* grp = getgrgid(pwd->pw_gid);
-			if (grp != NULL)
-				strlcpy(groupname, grp->gr_name, groupname_size);
-		}
-	}
-
-	return 0;
+	return get_user_auth_group(pctx->username, suggested, groupname, groupname_size);
 }
 
 static int pam_auth_user(void* ctx, char *username, int username_size)
@@ -356,33 +317,13 @@ struct pam_ctx_st * pctx = ctx;
 
 static void pam_group_list(void *pool, void *_additional, char ***groupname, unsigned *groupname_size)
 {
-	struct group *grp;
 	struct pam_cfg_st *config = _additional;
 	gid_t min = 0;
 
 	if (config)
 		min = config->gid_min;
 
-	setgrent();
-
-	*groupname_size = 0;
-	*groupname = talloc_size(pool, sizeof(char*)*MAX_GROUPS);
-	if (*groupname == NULL) {
-		goto exit;
-	}
-
-	while((grp = getgrent()) != NULL && (*groupname_size) < MAX_GROUPS) {
-		if (grp->gr_gid >= min) {
-			(*groupname)[(*groupname_size)] = talloc_strdup(*groupname, grp->gr_name);
-			if ((*groupname)[(*groupname_size)] == NULL)
-				break;
-			(*groupname_size)++;
-		}
-	}
-
- exit:
-	endgrent();
-	return;
+	unix_group_list(pool, min, groupname, groupname_size);
 }
 
 const struct auth_mod_st pam_auth_funcs = {
