@@ -12,6 +12,8 @@ enum {
 	CTL_CMD_ID_INFO,
 	CTL_CMD_DISCONNECT_NAME,
 	CTL_CMD_DISCONNECT_ID,
+	CTL_CMD_LIST_BANNED,
+	CTL_CMD_UNBAN_IP,
 
 	CTL_CMD_STATUS_REP = 101,
 	CTL_CMD_RELOAD_REP,
@@ -19,6 +21,8 @@ enum {
 	CTL_CMD_LIST_REP,
 	CTL_CMD_DISCONNECT_NAME_REP,
 	CTL_CMD_DISCONNECT_ID_REP,
+	CTL_CMD_UNBAN_IP_REP,
+	CTL_CMD_LIST_BANNED_REP,
 };
 
 #endif
