@@ -17,7 +17,6 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
- *
  */
 
 #ifndef GETTIME_H
@@ -32,7 +31,9 @@
 inline static void
 gettime (struct timespec *t)
 {
-#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
+#if defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME_COARSE)
+  clock_gettime (CLOCK_REALTIME_COARSE, t);
+#elif defined(HAVE_CLOCK_GETTIME) && defined(CLOCK_REALTIME)
   clock_gettime (CLOCK_REALTIME, t);
 #else
 struct timeval tv;
@@ -42,4 +43,12 @@ struct timeval tv;
 #endif
 }
 
+inline static
+unsigned int
+timespec_sub_ms (struct timespec *a, struct timespec *b)
+{
+  return (a->tv_sec * 1000 + a->tv_nsec / (1000 * 1000) -
+          (b->tv_sec * 1000 + b->tv_nsec / (1000 * 1000)));
+}
+
 #endif
