@@ -23,8 +23,13 @@
 
 # include "main.h"
 
+typedef struct inaddr_st {
+	uint8_t ip[16];
+	unsigned size; /* 4 or 16 */
+} inaddr_st;
+
 typedef struct ban_entry_st {
-	char ip[MAX_IP_STR];
+	inaddr_st ip;
 	unsigned score;
 
 	time_t last_reset; /* the time its score counting started */
@@ -33,8 +38,9 @@ typedef struct ban_entry_st {
 
 void cleanup_banned_entries(main_server_st *s);
 unsigned check_if_banned(main_server_st *s, struct sockaddr_storage *addr, socklen_t addr_size);
-int add_ip_to_ban_list(main_server_st *s, const char *ip, unsigned score);
-int remove_ip_from_ban_list(main_server_st *s, const char *ip);
+int add_ip_to_ban_list(main_server_st *s, const unsigned char *ip, unsigned ip_size, unsigned score);
+int add_str_ip_to_ban_list(main_server_st *s, const char *ip, unsigned score);
+int remove_ip_from_ban_list(main_server_st *s, const uint8_t *ip, unsigned size);
 unsigned main_ban_db_elems(main_server_st *s);
 void main_ban_db_deinit(main_server_st *s);
 void *main_ban_db_init(main_server_st *s);
