@@ -16,7 +16,7 @@ inline static void terminate_proc(main_server_st *s, proc_st *proc)
 	if (proc->pid != -1 && proc->pid != 0)
                 kill(proc->pid, SIGTERM);
 	else
-		remove_proc(s, proc, 1);
+		remove_proc(s, proc, RPROC_KILL);
 }
 
 #endif
