@@ -47,6 +47,7 @@
 #include <proc-search.h>
 #include <ipc.pb-c.h>
 #include <script-list.h>
+#include <inttypes.h>
 
 #ifdef HAVE_MALLOC_TRIM
 # include <malloc.h>
@@ -100,8 +101,6 @@ int handle_script_exit(main_server_st *s, struct proc_st *proc, int code)
 	int ret;
 
 	if (code == 0) {
-		proc->status = PS_AUTH_COMPLETED;
-
 		ret = send_cookie_auth_reply(s, proc, AUTH__REP__OK);
 		if (ret < 0) {
 			mslog(s, proc, LOG_ERR,
@@ -110,7 +109,15 @@ int handle_script_exit(main_server_st *s, struct proc_st *proc, int code)
 			goto fail;
 		}
 
-		apply_iroutes(s, proc);
+		ret = apply_iroutes(s, proc);
+		if (ret < 0) {
+			mslog(s, proc, LOG_ERR,
+			      "could not apply routes for user; denying access.");
+			ret = ERR_BAD_COMMAND;
+			goto fail;
+		}
+
+		proc->status = PS_AUTH_COMPLETED;
 	} else {
 		mslog(s, proc, LOG_INFO,
 		      "failed authentication attempt for user '%s'",
@@ -173,7 +180,7 @@ struct proc_st *ctmp;
  */
 void remove_proc(main_server_st * s, struct proc_st *proc, unsigned flags)
 {
-	mslog(s, proc, LOG_INFO, "user disconnected");
+	mslog(s, proc, LOG_INFO, "user disconnected (rx: %"PRIu64", tx: %"PRIu64")", proc->bytes_in, proc->bytes_out);
 
 	list_del(&proc->list);
 	s->active_clients--;
@@ -369,7 +376,7 @@ int handle_commands(main_server_st * s, struct proc_st *proc)
 				goto cleanup;
 			}
 
-			ret = add_ip_to_ban_list(s, tmsg->ip, tmsg->score);
+			ret = add_str_ip_to_ban_list(s, tmsg->ip, tmsg->score);
 
 			ban_ip_msg__free_unpacked(tmsg, &pa);
 
