@@ -493,6 +493,9 @@ struct script_wait_st *stmp = NULL, *spos;
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 		estatus = WEXITSTATUS(status);
 
+		if (WIFSIGNALED(status))
+			estatus = 1;
+
 		if (pid == s->sec_mod_pid) {
 			mslog(s, NULL, LOG_ERR, "ocserv-secmod died unexpectedly");
 			terminate = 1;
@@ -848,7 +851,7 @@ unsigned total = 10;
 	if (reload_conf != 0) {
 		mslog(s, NULL, LOG_INFO, "reloading configuration");
 		reload_cfg_file(s->main_pool, s->perm_config);
-		tls_reload_crl(s, s->creds);
+		tls_reload_crl(s, s->creds, 1);
 		reload_conf = 0;
 		kill(s->sec_mod_pid, SIGHUP);
 	}
@@ -889,6 +892,9 @@ unsigned total = 10;
 	if (need_maintenance != 0) {
 		need_maintenance = 0;
 		mslog(s, NULL, LOG_DEBUG, "performing maintenance (banned IPs: %d)", main_ban_db_elems(s));
+
+		/* will make sure it only reloads when needed */
+		tls_reload_crl(s, s->creds, 0);
 		expire_tls_sessions(s);
 		cleanup_banned_entries(s);
 		alarm(MAINTAINANCE_TIME(s));
@@ -1216,6 +1222,7 @@ int main(int argc, char** argv)
 
 					/* clear the cookie key */
 					safe_memset(s->cookie_key, 0, sizeof(s->cookie_key));
+					safe_memset(s->prev_cookie_key, 0, sizeof(s->prev_cookie_key));
 
 					setproctitle(PACKAGE_NAME"-worker");
 					kill_on_parent_kill(SIGTERM);
