@@ -179,6 +179,9 @@ typedef struct main_server_st {
 	tls_st *creds;
 	
 	uint8_t cookie_key[COOKIE_KEY_SIZE];
+	/* when we rotate keys, key the previous one active for verification */
+	uint8_t prev_cookie_key[COOKIE_KEY_SIZE];
+	unsigned prev_cookie_key_active;
 
 	struct listen_list_st listen_list;
 	struct proc_list_st proc_list;
@@ -236,19 +239,28 @@ int handle_resume_store_req(main_server_st* s, struct proc_st *proc,
 int session_open(main_server_st * s, struct proc_st *proc, const uint8_t *cookie, unsigned cookie_size);
 int session_close(main_server_st * s, struct proc_st *proc);
 
+#ifdef DISABLE_LOGS
+/* for testing */
+# define mslog(...)
+
+#else
+
 void 
 __attribute__ ((format(printf, 4, 5)))
     _mslog(const main_server_st * s, const struct proc_st* proc,
     	int priority, const char *fmt, ...);
 
-#ifdef __GNUC__
-# define mslog(s, proc, prio, fmt, ...) \
+# ifdef __GNUC__
+#  define mslog(s, proc, prio, fmt, ...) \
 	(prio==LOG_ERR)?_mslog(s, proc, prio, "%s:%d: "fmt, __FILE__, __LINE__, ##__VA_ARGS__): \
 	_mslog(s, proc, prio, fmt, ##__VA_ARGS__)
-#else
-# define mslog _mslog
+# else
+#  define mslog _mslog
+# endif
+
 #endif
 
+
 void  mslog_hex(const main_server_st * s, const struct proc_st* proc,
     	int priority, const char *prefix, uint8_t* bin, unsigned bin_size, unsigned b64);
 
