@@ -27,33 +27,34 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
+#include <occtl.h>
 
 #define _(x) x
 
 /* compact 7 char format for time intervals (belongs in libproc?) */
-void print_time_ival7(time_t t1, time_t t2, FILE * fout)
+void print_time_ival7(char output[MAX_TMPSTR_SIZE], time_t t1, time_t t2)
 {
 	time_t t = t1 - t2;
 
 	if ((long)t < (long)0) {
 		/* system clock changed? */
-		fprintf(fout, "   ?   ");
+		snprintf(output, MAX_TMPSTR_SIZE, "   ?   ");
 		return;
 	}
 	
 	if (t >= 48 * 60 * 60)
 		/* 2 days or more */
-		fprintf(fout, _("%2ludays"), t / (24 * 60 * 60));
+		snprintf(output, MAX_TMPSTR_SIZE, _("%2ludays"), (long)t / (24 * 60 * 60));
 	else if (t >= 60 * 60)
 		/* 1 hour or more */
 	        /* Translation Hint: Hours:Minutes */
-		fprintf(fout, _("%2luh:%02um"), t / (60 * 60),
+		snprintf(output, MAX_TMPSTR_SIZE, _("%2luh:%02um"), (long)t / (60 * 60),
 			(unsigned)((t / 60) % 60));
 	else if (t > 60)
 		/* 1 minute or more */
 	        /* Translation Hint: Minutes:Seconds */
-		fprintf(fout, "%2lum:%02us", t / 60, (unsigned)t % 60);
+		snprintf(output, MAX_TMPSTR_SIZE, "%2lum:%02us", (long)t / 60, (unsigned)t % 60);
 	else
 	        /* Translation Hint: Seconds:Centiseconds */
-		fprintf(fout, _("%5lus"), t);
+		snprintf(output, MAX_TMPSTR_SIZE, _("%5lus"), (long)t);
 }
