@@ -24,6 +24,9 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <limits.h>
+#include <signal.h>
+#include <unistd.h>
+#include <sys/types.h>
 #include <sys/wait.h>
 
 #include <route-add.h>
@@ -31,6 +34,51 @@
 #include <str.h>
 #include <common.h>
 
+static
+int call_script(main_server_st *s, proc_st *proc, const char *cmd)
+{
+pid_t pid;
+int ret, status = 0;
+
+	if (cmd == NULL)
+		return 0;
+
+	pid = fork();
+	if (pid == 0) {
+		sigprocmask(SIG_SETMASK, &sig_default_set, NULL);
+
+		mslog(s, proc, LOG_DEBUG, "executing route script %s", cmd);
+		ret = execl("/bin/sh", "sh", "-c", cmd, NULL);
+		if (ret == -1) {
+			mslog(s, proc, LOG_ERR, "Could not execute route script %s", cmd);
+			exit(1);
+		}
+
+		exit(77);
+	} else if (pid == -1) {
+		mslog(s, proc, LOG_ERR, "Could not fork()");
+		return ERR_EXEC;
+	}
+
+	ret = waitpid(pid, &status, 0);
+	if (ret == -1) {
+		mslog(s, proc, LOG_ERR, "Could not waitpid()");
+		return ERR_EXEC;
+	}
+
+	if (!WIFEXITED(status)) {
+		mslog(s, proc, LOG_INFO, "cmd: %s: exited abnormally", cmd);
+		return ERR_EXEC;
+	}
+
+	if (WEXITSTATUS(status)) {
+		mslog(s, proc, LOG_INFO, "cmd: %s: exited with error %d", cmd, WEXITSTATUS(ret));
+		return ERR_EXEC;
+	}
+
+	return 0;
+}
+
 static
 int replace_cmd(struct main_server_st* s, proc_st *proc, 
 		char **cmd, const char* pattern, 
@@ -38,6 +86,14 @@ int replace_cmd(struct main_server_st* s, proc_st *proc,
 {
 	str_st str;
 	int ret;
+	str_rep_tab tab[6];
+
+	STR_TAB_SET(0, "%{R}", route);
+	STR_TAB_SET(1, "%R", route);
+	STR_TAB_SET(2, "%{D}", dev);
+	STR_TAB_SET(3, "%D", dev);
+	STR_TAB_SET_FUNC(4, "%{RI}", ipv4_route_to_cidr, route);
+	STR_TAB_TERM(5);
 
 	str_init(&str, proc);
 
@@ -45,20 +101,7 @@ int replace_cmd(struct main_server_st* s, proc_st *proc,
 	if (ret < 0)
 		return ERR_MEM;
 
-	ret = str_replace_str(&str, "%{R}", route);
-	if (ret < 0)
-		goto fail;
-
-	ret = str_replace_str(&str, "%{D}", dev);
-	if (ret < 0)
-		goto fail;
-
-	/* The old compatibility strings */
-	ret = str_replace_str(&str, "%R", route);
-	if (ret < 0)
-		goto fail;
-
-	ret = str_replace_str(&str, "%D", dev);
+	ret = str_replace_str(&str, tab);
 	if (ret < 0)
 		goto fail;
 
@@ -85,28 +128,15 @@ char *cmd = NULL;
 	ret = replace_cmd(s, proc, &cmd, pattern, route, dev);
 	if (ret < 0)
 		return ret;
-	
-	mslog(s, NULL, LOG_DEBUG, "spawning cmd: %s", cmd);
-	ret = system(cmd);
-	if (ret == -1) {
+
+	ret = call_script(s, proc, cmd);
+	if (ret < 0) {
 		int e = errno;
 		mslog(s, NULL, LOG_INFO, "failed to spawn cmd: %s: %s", cmd, strerror(e));
 		ret = ERR_EXEC;
 		goto fail;
 	}
-	
-	if (!WIFEXITED(ret)) {
-		mslog(s, NULL, LOG_INFO, "cmd: %s: exited abnormally", cmd);
-		ret = ERR_EXEC;
-		goto fail;
-	}
 
-	if (WEXITSTATUS(ret)) {
-		mslog(s, NULL, LOG_INFO, "cmd: %s: exited with error %d", cmd, WEXITSTATUS(ret));
-		ret = ERR_EXEC;
-		goto fail;
-	}
-	
 	ret = 0;
  fail:
  	talloc_free(cmd);
@@ -128,13 +158,13 @@ int route_del(struct main_server_st* s, proc_st *proc, const char* route, const
 /* Executes the commands required to apply all the configured routes 
  * for this client locally.
  */
-void apply_iroutes(struct main_server_st* s, struct proc_st *proc)
+int apply_iroutes(struct main_server_st* s, struct proc_st *proc)
 {
 unsigned i, j;
 int ret;
 
 	if (proc->config.iroutes_size == 0)
-		return;
+		return 0;
 
 	for (i=0;i<proc->config.iroutes_size;i++) {
 		ret = route_add(s, proc, proc->config.iroutes[i], proc->tun_lease.name);
@@ -142,13 +172,13 @@ int ret;
 			goto fail;
 	}
 	proc->applied_iroutes = 1;
-	
-	return;
+
+	return 0;
 fail:
 	for (j=0;j<i;j++)
 		route_del(s, proc, proc->config.iroutes[j], proc->tun_lease.name);
-	
-	return;
+
+	return -1;
 }
 
 /* Executes the commands required to removed all the configured routes 
@@ -165,7 +195,7 @@ unsigned i;
 		route_del(s, proc, proc->config.iroutes[i], proc->tun_lease.name);
 	}
 	proc->applied_iroutes = 0;
-	
+
 	return;
 }
 
