@@ -83,7 +83,7 @@ struct htable *db = sec->client_db;
 		return 0;
 }
 
-client_entry_st *new_client_entry(sec_mod_st *sec, const char *ip)
+client_entry_st *new_client_entry(sec_mod_st *sec, const char *ip, unsigned pid)
 {
 	struct htable *db = sec->client_db;
 	client_entry_st *e, *te;
@@ -96,6 +96,7 @@ client_entry_st *new_client_entry(sec_mod_st *sec, const char *ip)
 	}
 
 	strlcpy(e->auth_info.remote_ip, ip, sizeof(e->auth_info.remote_ip));
+	e->auth_info.id = pid;
 
 	do {
 		ret = gnutls_rnd(GNUTLS_RND_RANDOM, e->sid, sizeof(e->sid));
@@ -188,6 +189,17 @@ void expire_client_entry(sec_mod_st *sec, client_entry_st * e)
 {
 	if (e->in_use > 0)
 		e->in_use--;
-	if (e->in_use == 0)
+	if (e->in_use == 0) {
 		e->time = time(0);
+
+		if (sec->config->persistent_cookies == 0 && (e->discon_reason == REASON_USER_DISCONNECT
+		    || e->discon_reason == REASON_SERVER_DISCONNECT || e->discon_reason == REASON_SESSION_TIMEOUT)) {
+			seclog(sec, LOG_INFO, "invalidating session of user '%s' "SESSION_STR,
+				e->auth_info.username, e->auth_info.psid);
+			/* immediately disconnect the user */
+			del_client_entry(sec, e);
+		} else {
+			seclog(sec, LOG_INFO, "temporarily closing session for %s "SESSION_STR, e->auth_info.username, e->auth_info.psid);
+		}
+	}
 }
