@@ -30,7 +30,7 @@
  * proc->username/proc->groupname and save it in proc->config.
  */
 struct config_mod_st {
-	int (*get_sup_config)(struct cfg_st *global_config, client_entry_st *entry,
+	int (*get_sup_config)(struct cfg_st *perm_config, client_entry_st *entry,
 	                      SecAuthSessionReplyMsg *msg, void *pool);
 };
 
