@@ -62,7 +62,7 @@ typedef struct tls_st {
 	gnutls_dh_params_t dh_params;
 } tls_st;
 
-void tls_reload_crl(struct main_server_st* s, struct tls_st *creds);
+void tls_reload_crl(struct main_server_st* s, struct tls_st *creds, unsigned force);
 void tls_global_init(struct tls_st *creds);
 void tls_global_deinit(struct tls_st *creds);
 void tls_load_certs(struct main_server_st* s, struct tls_st *creds);
