@@ -66,6 +66,7 @@
 
 #define PERIODIC_CHECK_TIME 30
 #define MIN_STATS_INTERVAL 10
+#define SOCKET_TIMEO_SECS 10
 
 /* The number of DPD packets a client skips before he's kicked */
 #define DPD_TRIES 2
@@ -306,9 +307,6 @@ void send_stats_to_secmod(worker_st * ws, time_t now, unsigned discon_reason)
 	CliStatsMsg msg = CLI_STATS_MSG__INIT;
 	int sd, ret, e;
 
-	if (now - ws->last_stats_msg < MIN_STATS_INTERVAL)
-		return;
-
 	ws->last_stats_msg = now;
 
 	sd = connect_to_secmod(ws);
@@ -338,7 +336,7 @@ void send_stats_to_secmod(worker_st * ws, time_t now, unsigned discon_reason)
 		close(sd);
 
 		if (ret >= 0) {
-			oclog(ws, LOG_DEBUG,
+			oclog(ws, LOG_INFO,
 			      "sent periodic stats (in: %lu, out: %lu) to sec-mod",
 			      (unsigned long)msg.bytes_in,
 			      (unsigned long)msg.bytes_out);
@@ -682,7 +680,7 @@ int mtu_not_ok(worker_st * ws)
 		}
 
 		mtu_set(ws, ws->last_good_mtu);
-		oclog(ws, LOG_DEBUG, "MTU %u is too large, switching to %u",
+		oclog(ws, LOG_INFO, "MTU %u is too large, switching to %u",
 		      ws->last_bad_mtu, ws->conn_mtu);
 	} else if (ws->proto == AF_INET6) { /* IPv6 */
 		int mtu;
@@ -1175,7 +1173,7 @@ static int tun_mainloop(struct worker_st *ws, struct timespec *tnow)
 	if (ws->udp_state == UP_ACTIVE && ws->dtls_selected_comp != NULL && l > ws->config->no_compress_limit) {
 		/* otherwise don't compress */
 		ret = ws->dtls_selected_comp->compress(ws->decomp+8, sizeof(ws->decomp)-8, ws->buffer+8, l);
-		oclog(ws, LOG_DEBUG, "compressed %d to %d\n", (int)l, ret);
+		oclog(ws, LOG_TRANSFER_DEBUG, "compressed %d to %d\n", (int)l, ret);
 		if (ret > 0 && ret < l) {
 			dtls_to_send.data = ws->decomp;
 			dtls_to_send.size = ret;
@@ -1192,7 +1190,7 @@ static int tun_mainloop(struct worker_st *ws, struct timespec *tnow)
 	} else if (ws->cstp_selected_comp != NULL && l > ws->config->no_compress_limit) {
 		/* otherwise don't compress */
 		ret = ws->cstp_selected_comp->compress(ws->decomp+8, sizeof(ws->decomp)-8, ws->buffer+8, l);
-		oclog(ws, LOG_DEBUG, "compressed %d to %d\n", (int)l, ret);
+		oclog(ws, LOG_TRANSFER_DEBUG, "compressed %d to %d\n", (int)l, ret);
 		if (ret > 0 && ret < l) {
 			cstp_to_send.data = ws->decomp;
 			cstp_to_send.size = ret;
@@ -1253,6 +1251,11 @@ char *replace_vals(worker_st *ws, const char *txt)
 {
 	str_st str;
 	int ret;
+	str_rep_tab tab[3];
+
+	STR_TAB_SET(0, "%{U}", ws->username);
+	STR_TAB_SET(1, "%{G}", ws->groupname);
+	STR_TAB_TERM(2);
 
 	str_init(&str, ws);
 
@@ -1260,13 +1263,7 @@ char *replace_vals(worker_st *ws, const char *txt)
 	if (ret < 0)
 		return NULL;
 
-	ret = str_replace_str(&str, "%{U}", ws->username);
-	if (ret < 0) {
-		str_clear(&str);
-		return NULL;
-	}
-
-	ret = str_replace_str(&str, "%{G}", ws->groupname);
+	ret = str_replace_str(&str, tab);
 	if (ret < 0) {
 		str_clear(&str);
 		return NULL;
@@ -1299,7 +1296,7 @@ static int send_routes(worker_st *ws, struct http_req_st *req,
 			continue;
 		if (req->no_ipv4 != 0 && ip6 == 0)
 			continue;
-		oclog(ws, LOG_DEBUG, "%s route %s", txt, routes[i]);
+		oclog(ws, LOG_INFO, "%s route %s", txt, routes[i]);
 
 		if (ip6 != 0 && ws->full_ipv6) {
 			ret = cstp_printf(ws,
@@ -1316,6 +1313,25 @@ static int send_routes(worker_st *ws, struct http_req_st *req,
 	return 0;
 }
 
+/* Enforces a socket timeout. That is because, although we
+ * use select() to see whether a call to recv() would block,
+ * there are certain cases in Linux where recv() blocks even
+ * though select() notified of data */
+static void set_socket_timeout(worker_st * ws, int fd)
+{
+	struct timeval tval;
+	int ret;
+
+	tval.tv_sec = SOCKET_TIMEO_SECS;
+	tval.tv_usec = 0;
+	ret =
+	    setsockopt(fd, SOL_SOCKET, SO_RCVTIMEO, &tval,
+			       sizeof(tval));
+	if (ret == -1)
+		oclog(ws, LOG_DEBUG,
+		      "setsockopt(%s, SO_RCVTIMEO) failed.", (fd==ws->conn_fd)?"ΤCP":"UDP");
+}
+
 /* connect_handler:
  * @ws: an initialized worker structure
  *
@@ -1431,7 +1447,7 @@ static int connect_handler(worker_st * ws)
 		ws->config->idle_timeout = ws->config->mobile_idle_timeout;
 	}
 
-	oclog(ws, LOG_DEBUG, "suggesting DPD of %d secs", ws->config->dpd);
+	oclog(ws, LOG_INFO, "suggesting DPD of %d secs", ws->config->dpd);
 	if (ws->config->dpd > 0) {
 		ret =
 		    cstp_printf(ws, "X-CSTP-DPD: %u\r\n",
@@ -1460,7 +1476,7 @@ static int connect_handler(worker_st * ws)
 	}
 
 	if (req->base_mtu > 0) {
-		oclog(ws, LOG_DEBUG, "peer's base MTU is %u", req->base_mtu);
+		oclog(ws, LOG_INFO, "peer's base MTU is %u", req->base_mtu);
 		ws->vinfo.mtu = MIN(ws->vinfo.mtu, req->base_mtu);
 	}
 
@@ -1473,10 +1489,9 @@ static int connect_handler(worker_st * ws)
 			      strerror(e));
 		} else {
 			max -= 13;
-			oclog(ws, LOG_DEBUG, "TCP MSS is %u", max);
 			if (max > 0 && max < ws->vinfo.mtu) {
-				oclog(ws, LOG_DEBUG,
-				      "reducing MTU due to TCP MSS to %u", max);
+				oclog(ws, LOG_INFO,
+				      "reducing MTU due to TCP MSS to %u (from %u)", max, ws->vinfo.mtu);
 				ws->vinfo.mtu = max;
 			}
 		}
@@ -1484,7 +1499,7 @@ static int connect_handler(worker_st * ws)
 
 	ret = cstp_printf(ws, "X-CSTP-Base-MTU: %u\r\n", ws->vinfo.mtu);
 	SEND_ERR(ret);
-	oclog(ws, LOG_DEBUG, "CSTP Base MTU is %u bytes", ws->vinfo.mtu);
+	oclog(ws, LOG_INFO, "CSTP Base MTU is %u bytes", ws->vinfo.mtu);
 
 	/* calculate TLS channel MTU */
 	if (ws->session == NULL) {
@@ -1508,7 +1523,7 @@ static int connect_handler(worker_st * ws)
 
 	/* Send IP addresses */
 	if (ws->vinfo.ipv4 && req->no_ipv4 == 0) {
-		oclog(ws, LOG_DEBUG, "sending IPv4 %s", ws->vinfo.ipv4);
+		oclog(ws, LOG_INFO, "sending IPv4 %s", ws->vinfo.ipv4);
 		ret =
 		    cstp_printf(ws, "X-CSTP-Address: %s\r\n",
 			       ws->vinfo.ipv4);
@@ -1523,7 +1538,7 @@ static int connect_handler(worker_st * ws)
 	}
 
 	if (ws->vinfo.ipv6 && req->no_ipv6 == 0 && ws->vinfo.ipv6_prefix != 0) {
-		oclog(ws, LOG_DEBUG, "sending IPv6 %s/%u", ws->vinfo.ipv6, ws->vinfo.ipv6_prefix);
+		oclog(ws, LOG_INFO, "sending IPv6 %s/%u", ws->vinfo.ipv6, ws->vinfo.ipv6_prefix);
 		if (ws->full_ipv6 && ws->vinfo.ipv6_prefix) {
 			ret =
 			    cstp_printf(ws,
@@ -1566,6 +1581,7 @@ static int connect_handler(worker_st * ws)
 		if (req->no_ipv4 != 0 && ip6 == 0)
 			continue;
 
+		oclog(ws, LOG_INFO, "adding DNS %s", ws->vinfo.dns[i]);
 		ret =
 		    cstp_printf(ws, "X-CSTP-DNS: %s\r\n",
 			       ws->vinfo.dns[i]);
@@ -1583,6 +1599,7 @@ static int connect_handler(worker_st * ws)
 		if (req->no_ipv4 != 0 && ip6 == 0)
 			continue;
 
+		oclog(ws, LOG_INFO, "adding NBNS %s", ws->vinfo.nbns[i]);
 		ret =
 		    cstp_printf(ws, "X-CSTP-NBNS: %s\r\n",
 			       ws->vinfo.nbns[i]);
@@ -1600,7 +1617,7 @@ static int connect_handler(worker_st * ws)
 		if (req->no_ipv4 != 0 && ip6 == 0)
 			continue;
 
-		oclog(ws, LOG_DEBUG, "adding split DNS %s",
+		oclog(ws, LOG_INFO, "adding split DNS %s",
 		      ws->config->split_dns[i]);
 		ret =
 		    cstp_printf(ws, "X-CSTP-Split-DNS: %s\r\n",
@@ -1614,8 +1631,19 @@ static int connect_handler(worker_st * ws)
 
 		ret = send_routes(ws, req, ws->routes, ws->routes_size, 1);
 		SEND_ERR(ret);
+
+	} else {
+		/* default route */
+		ws->config->tunnel_all_dns = 1;
 	}
 
+	if (ws->config->tunnel_all_dns) {
+		ret = cstp_puts(ws, "X-CSTP-Tunnel-All-DNS: true\r\n");
+	} else {
+		ret = cstp_puts(ws, "X-CSTP-Tunnel-All-DNS: false\r\n");
+	}
+	SEND_ERR(ret);
+
 	ret = send_routes(ws, req, ws->vinfo.no_routes, ws->vinfo.no_routes_size, 0);
 	SEND_ERR(ret);
 
@@ -1688,7 +1716,6 @@ static int connect_handler(worker_st * ws)
 		       "X-CSTP-Disconnected-Timeout: none\r\n"
 		       "X-CSTP-Keep: true\r\n"
 		       "X-CSTP-TCP-Keepalive: true\r\n"
-		       "X-CSTP-Tunnel-All-DNS: false\r\n"
 		       "X-CSTP-License: accept\r\n");
 	SEND_ERR(ret);
 
@@ -1696,7 +1723,7 @@ static int connect_handler(worker_st * ws)
 		char *h = replace_vals(ws, ws->config->custom_header[i]);
 
 		if (h) {
-			oclog(ws, LOG_DEBUG, "adding custom header '%s'", h);
+			oclog(ws, LOG_INFO, "adding custom header '%s'", h);
 			ret =
 			    cstp_printf(ws, "%s\r\n", h);
 			SEND_ERR(ret);
@@ -1716,6 +1743,7 @@ static int connect_handler(worker_st * ws)
 			      "setsockopt(TCP, SO_SNDBUF) to %u, failed.", t);
 	}
 
+	set_socket_timeout(ws, ws->conn_fd);
 	set_non_block(ws->conn_fd);
 	set_net_priority(ws, ws->conn_fd, ws->config->net_priority);
 
@@ -1763,7 +1791,7 @@ static int connect_handler(worker_st * ws)
 			       ws->config->keepalive);
 		SEND_ERR(ret);
 
-		oclog(ws, LOG_DEBUG, "DTLS ciphersuite: %s",
+		oclog(ws, LOG_INFO, "DTLS ciphersuite: %s",
 		      ws->req.selected_ciphersuite->oc_name);
 		ret =
 		    cstp_printf(ws, "X-DTLS-CipherSuite: %s\r\n",
@@ -1800,7 +1828,7 @@ static int connect_handler(worker_st * ws)
 		ret =
 		    cstp_printf(ws, "X-DTLS-MTU: %u\r\n", ws->conn_mtu);
 		SEND_ERR(ret);
-		oclog(ws, LOG_DEBUG, "suggesting DTLS MTU %u", ws->conn_mtu);
+		oclog(ws, LOG_INFO, "suggesting DTLS MTU %u", ws->conn_mtu);
 
 		if (ws->config->output_buffer > 0) {
 			t = MIN(2048, ws->conn_mtu * ws->config->output_buffer);
@@ -1813,6 +1841,7 @@ static int connect_handler(worker_st * ws)
 		}
 
 		set_net_priority(ws, ws->dtls_tptr.fd, ws->config->net_priority);
+		set_socket_timeout(ws, ws->dtls_tptr.fd);
 	}
 
 	/* hack for openconnect. It uses only a single MTU value */
@@ -1837,7 +1866,7 @@ static int connect_handler(worker_st * ws)
 
 	/* send any compression methods */
 	if (ws->dtls_selected_comp) {
-		oclog(ws, LOG_DEBUG, "selected DTLS compression method %s\n", ws->dtls_selected_comp->name);
+		oclog(ws, LOG_INFO, "selected DTLS compression method %s\n", ws->dtls_selected_comp->name);
 		ret =
 		    cstp_printf(ws, "X-DTLS-Content-Encoding: %s\r\n",
 			        ws->dtls_selected_comp->name);
@@ -1845,7 +1874,7 @@ static int connect_handler(worker_st * ws)
 	}
 
 	if (ws->cstp_selected_comp) {
-		oclog(ws, LOG_DEBUG, "selected CSTP compression method %s\n", ws->cstp_selected_comp->name);
+		oclog(ws, LOG_INFO, "selected CSTP compression method %s\n", ws->cstp_selected_comp->name);
 		ret =
 		    cstp_printf(ws, "X-CSTP-Content-Encoding: %s\r\n",
 			        ws->cstp_selected_comp->name);
@@ -2066,7 +2095,7 @@ static int parse_data(struct worker_st *ws, uint8_t *buf, size_t buf_size,
 
 		break;
 	case AC_PKT_DISCONN:
-		oclog(ws, LOG_DEBUG, "received BYE packet; exiting");
+		oclog(ws, LOG_INFO, "received BYE packet; exiting");
 		exit_worker_reason(ws, REASON_USER_DISCONNECT);
 		break;
 	case AC_PKT_COMPRESSED:
